@@ -43,44 +43,52 @@
 #include <libxml/tree.h>
 #endif
 
+#define BAD_CAST_CONST (const xmlChar *)
+
+
 #ifdef HAVE_LIBXML2
 static void _parseManifestNode(xmlNode * a_node, META_DL *disclib)
 {
     xmlNode *cur_node = NULL;
+    xmlChar *tmp;
 
     for (cur_node = a_node; cur_node; cur_node = cur_node->next) {
         if (cur_node->type == XML_ELEMENT_NODE) {
-            if (xmlStrEqual(cur_node->parent->name, xmlCharStrdup("title"))) {
-                if (xmlStrEqual(cur_node->name, xmlCharStrdup("name"))) {
-                    disclib->di_name = (char*)xmlStrdup(xmlNodeGetContent(cur_node));
+            if (xmlStrEqual(cur_node->parent->name, BAD_CAST_CONST "title")) {
+                if (xmlStrEqual(cur_node->name, BAD_CAST_CONST "name")) {
+                    disclib->di_name = (char*)xmlNodeGetContent(cur_node);
                 }
-                if (xmlStrEqual(cur_node->name, xmlCharStrdup("alternative"))) {
-                    disclib->di_alternative = (char*)xmlStrdup(xmlNodeGetContent(cur_node));
+                if (xmlStrEqual(cur_node->name, BAD_CAST_CONST "alternative")) {
+                    disclib->di_alternative = (char*)xmlNodeGetContent(cur_node);
                 }
-                if (xmlStrEqual(cur_node->name, xmlCharStrdup("numSets"))) {
-                    disclib->di_num_sets = atoi((char*)xmlNodeGetContent(cur_node));
+                if (xmlStrEqual(cur_node->name, BAD_CAST_CONST "numSets")) {
+                    disclib->di_num_sets = atoi((char*)(tmp = xmlNodeGetContent(cur_node)));
+                    xmlFree(tmp);
                 }
-                if (xmlStrEqual(cur_node->name, xmlCharStrdup("setNumber"))) {
-                    disclib->di_set_number = atoi((char*)xmlNodeGetContent(cur_node));
+                if (xmlStrEqual(cur_node->name, BAD_CAST_CONST "setNumber")) {
+                    disclib->di_set_number = atoi((char*)(tmp = xmlNodeGetContent(cur_node)));
+                    xmlFree(tmp);
                 }
             }
-            else if (xmlStrEqual(cur_node->parent->name, xmlCharStrdup("tableOfContents"))) {
-                if (xmlStrEqual(cur_node->name, xmlCharStrdup("titleName")) && xmlGetProp(cur_node, xmlCharStrdup("titleNumber"))) {
+            else if (xmlStrEqual(cur_node->parent->name, BAD_CAST_CONST "tableOfContents")) {
+                if (xmlStrEqual(cur_node->name, BAD_CAST_CONST "titleName") && (tmp = xmlGetProp(cur_node, BAD_CAST_CONST "titleNumber"))) {
                     int i = disclib->toc_count;
                     disclib->toc_count++;
                     disclib->toc_entries = realloc(disclib->toc_entries, (disclib->toc_count*sizeof(META_TITLE)));
-                    disclib->toc_entries[i].title_number = atoi((const char*)xmlGetProp(cur_node, xmlCharStrdup("titleNumber")));
-                    disclib->toc_entries[i].title_name = (char*)xmlStrdup(xmlNodeGetContent(cur_node));
+                    disclib->toc_entries[i].title_number = atoi((const char*)tmp);
+                    disclib->toc_entries[i].title_name = (char*)xmlNodeGetContent(cur_node);
+                    X_FREE(tmp);
                 }
             }
-            else if (xmlStrEqual(cur_node->parent->name, xmlCharStrdup("description"))) {
-                if (xmlStrEqual(cur_node->name, xmlCharStrdup("thumbnail")) && xmlGetProp(cur_node, xmlCharStrdup("href"))) {
+            else if (xmlStrEqual(cur_node->parent->name, BAD_CAST_CONST "description")) {
+                if (xmlStrEqual(cur_node->name, BAD_CAST_CONST "thumbnail") && (tmp = xmlGetProp(cur_node, BAD_CAST_CONST "href"))) {
                     uint8_t i = disclib->thumb_count;
                     disclib->thumb_count++;
                     disclib->thumbnails = realloc(disclib->thumbnails, (disclib->thumb_count*sizeof(META_THUMBNAIL)));
-                    disclib->thumbnails[i].path = strdup((const char*)xmlGetProp(cur_node, xmlCharStrdup("href")));
-                    if (xmlGetProp(cur_node, xmlCharStrdup("size"))) {
-                        sscanf((const char*)xmlGetProp(cur_node, xmlCharStrdup("size")), "%ix%i", &disclib->thumbnails[i].xres, &disclib->thumbnails[i].yres);
+                    disclib->thumbnails[i].path = (char *)tmp;
+                    if ((tmp = xmlGetProp(cur_node, BAD_CAST_CONST "size"))) {
+                        sscanf((const char*)tmp, "%ix%i", &disclib->thumbnails[i].xres, &disclib->thumbnails[i].yres);
+                        X_FREE(tmp);
                     }
                     else {
                         disclib->thumbnails[i].xres = disclib->thumbnails[i].yres = -1;
@@ -142,6 +150,8 @@ META_ROOT *meta_parse(const char *device_path)
         BD_FILE_H *handle = file_open(path, "rb");
         if (handle == NULL) {
             BD_DEBUG(DBG_DIR, "Failed to open meta file (%s)\n", path);
+            X_FREE(path);
+            X_FREE(base);
             continue;
         }
 
@@ -155,6 +165,8 @@ META_ROOT *meta_parse(const char *device_path)
             doc = xmlReadMemory((char*)data, size_read, base, NULL, 0);
             if (doc == NULL) {
                 BD_DEBUG(DBG_DIR, "Failed to parse %s\n", path);
+                X_FREE(path);
+                X_FREE(base);
                 continue;
             }
             xmlNode *root_element = NULL;
@@ -169,6 +181,8 @@ META_ROOT *meta_parse(const char *device_path)
             X_FREE(data);
         }
         file_close(handle);
+        X_FREE(path);
+        X_FREE(base);
     }
     xmlCleanupParser();
     return root;
@@ -214,18 +228,19 @@ void meta_free(META_ROOT **p)
         uint8_t i;
         for (i = 0; i < (*p)->dl_count; i++) {
             uint32_t t;
-            for (t=0; i < (*p)->dl_entries[i].toc_count; t++) {
+            for (t = 0; t < (*p)->dl_entries[i].toc_count; t++) {
                 X_FREE((*p)->dl_entries[i].toc_entries[t].title_name);
-                X_FREE((*p)->dl_entries[i].toc_entries);
             }
-            for (t = 0; i < (*p)->dl_entries[i].thumb_count; t++) {
+            for (t = 0; t < (*p)->dl_entries[i].thumb_count; t++) {
                 X_FREE((*p)->dl_entries[i].thumbnails[t].path);
-                X_FREE((*p)->dl_entries[i].thumbnails);
             }
+            X_FREE((*p)->dl_entries[i].toc_entries);
+            X_FREE((*p)->dl_entries[i].thumbnails);
             X_FREE((*p)->dl_entries[i].filename);
             X_FREE((*p)->dl_entries[i].di_name);
             X_FREE((*p)->dl_entries[i].di_alternative);
         }
+        X_FREE((*p)->dl_entries);
         X_FREE(*p);
     }
 }
