@@ -258,8 +258,9 @@ static int _queue_event(BLURAY *bd, BD_EVENT ev)
 
 static void _update_stream_psr_by_lang(BD_REGISTERS *regs,
                                        uint32_t psr_lang, uint32_t psr_stream,
-                                       uint32_t enable_flag, uint32_t undefined_val,
-                                       MPLS_STREAM *streams, unsigned num_streams)
+                                       uint32_t enable_flag,
+                                       MPLS_STREAM *streams, unsigned num_streams,
+                                       uint32_t *lang, uint32_t blacklist)
 {
     uint32_t psr_val;
     int      stream_idx = -1;
@@ -283,13 +284,21 @@ static void _update_stream_psr_by_lang(BD_REGISTERS *regs,
 
     if (stream_idx < 0) {
         /* requested language not found */
-        stream_idx = undefined_val - 1;
+        stream_idx = 0;
         enable_flag = 0;
+
+    } else {
+        if (lang) {
+            *lang = psr_val;
+        }
+        if (blacklist == psr_val) {
+            enable_flag = 0;
+        }
     }
 
     /* update PSR */
 
-    BD_DEBUG(DBG_BLURAY, "Selected stream %d (language %s)\n", ii, streams[ii].lang);
+    BD_DEBUG(DBG_BLURAY, "Selected stream %d (language %s)\n", stream_idx, streams[stream_idx].lang);
 
     bd_psr_lock(regs);
 
@@ -310,13 +319,16 @@ static void _update_clip_psrs(BLURAY *bd, NAV_CLIP *clip)
      */
     if (bd->title_type == title_undef) {
         MPLS_STN *stn = &clip->title->pl->play_item[clip->ref].stn;
+        uint32_t audio_lang = 0;
 
         _update_stream_psr_by_lang(bd->regs,
-                                   PSR_AUDIO_LANG, PSR_PRIMARY_AUDIO_ID, 0, 0xff,
-                                   stn->audio, stn->num_audio);
+                                   PSR_AUDIO_LANG, PSR_PRIMARY_AUDIO_ID, 0,
+                                   stn->audio, stn->num_audio,
+                                   &audio_lang, 0);
         _update_stream_psr_by_lang(bd->regs,
-                                   PSR_PG_AND_SUB_LANG, PSR_PG_STREAM, 0x80000000, 0xfff,
-                                   stn->pg, stn->num_pg);
+                                   PSR_PG_AND_SUB_LANG, PSR_PG_STREAM, 0x80000000,
+                                   stn->pg, stn->num_pg,
+                                   NULL, audio_lang);
     }
 }
 
@@ -701,17 +713,59 @@ static int _libaacs_open(BLURAY *bd, const char *keyfile_path)
         return 0;
     }
 
-    bd->aacs = bd->libaacs_open(bd->device_path, keyfile_path);
+    int error_code = 0;
+    fptr_p_void aacs_open2 = (fptr_p_void)dl_dlsym(bd->h_libaacs, "aacs_open2");
+    if (!aacs_open2) {
+        BD_DEBUG(DBG_BLURAY, "Using old aacs_open(), no verbose error reporting available (%p)\n", bd->aacs);
+        bd->aacs = bd->libaacs_open(bd->device_path, keyfile_path);
+    } else {
+        bd->aacs = aacs_open2(bd->device_path, keyfile_path, &error_code);
+    }
 
     if (bd->aacs) {
-        BD_DEBUG(DBG_BLURAY, "Opened libaacs (%p)\n", bd->aacs);
-        bd->disc_info.aacs_handled = 1;
-        return 1;
+        fptr_int    aacs_get_mkb_version = (fptr_int)    dl_dlsym(bd->h_libaacs, "aacs_get_mkb_version");
+        fptr_p_void aacs_get_disc_id     = (fptr_p_void) dl_dlsym(bd->h_libaacs, "aacs_get_disc_id");
+        if (aacs_get_mkb_version) {
+            bd->disc_info.aacs_mkbv = aacs_get_mkb_version(bd->aacs);
+        }
+        if (aacs_get_disc_id) {
+            memcpy(bd->disc_info.disc_id, aacs_get_disc_id(bd->aacs), 20);
+        }
+
+        if (!error_code) {
+            BD_DEBUG(DBG_BLURAY, "Opened libaacs (%p)\n", bd->aacs);
+            bd->disc_info.aacs_handled = 1;
+            return 1;
+        }
     }
 
     BD_DEBUG(DBG_BLURAY, "aacs_open() failed!\n");
     bd->disc_info.aacs_handled = 0;
 
+    switch (error_code) {
+        case 0: /* AACS_SUCCESS */
+            break;
+        case -1: /* AACS_ERROR_CORRUPTED_DISC */
+            bd->disc_info.aacs_error_code = BD_AACS_CORRUPTED_DISC;
+            break;
+        case -2: /* AACS_ERROR_NO_CONFIG */
+            bd->disc_info.aacs_error_code = BD_AACS_NO_CONFIG;
+            break;
+        case -3: /* AACS_ERROR_NO_PK */
+            bd->disc_info.aacs_error_code = BD_AACS_NO_PK;
+            break;
+        case -4: /* AACS_ERROR_NO_CERT */
+            bd->disc_info.aacs_error_code = BD_AACS_NO_CERT;
+            break;
+        case -5: /* AACS_ERROR_CERT_REVOKED */
+            bd->disc_info.aacs_error_code = BD_AACS_CERT_REVOKED;
+            break;
+        case -6: /* AACS_ERROR_MMC_OPEN */
+        case -7: /* AACS_ERROR_MMC_FAILURE */
+            bd->disc_info.aacs_error_code = BD_AACS_MMC_FAILED;
+            break;
+    }
+
     _libaacs_unload(bd);
     return 0;
 }
@@ -1048,6 +1102,7 @@ void bd_close(BLURAY *bd)
 
     gc_free(&bd->graphics_controller);
     indx_free(&bd->index);
+    meta_free(&bd->meta);
     sound_free(&bd->sound_effects);
     bd_registers_free(bd->regs);
 
@@ -1307,15 +1362,15 @@ int bd_read(BLURAY *bd, unsigned char *buf, int len)
                     }
                 }
             }
+            if (st->clip == NULL) {
+                // We previously reached the last clip.  Nothing
+                // else to read.
+                _queue_event(bd, (BD_EVENT){BD_EVENT_END_OF_TITLE, 0});
+                return 0;
+            }
             if (st->int_buf_off == 6144 || clip_pkt >= st->clip->end_pkt) {
 
                 // Do we need to get the next clip?
-                if (st->clip == NULL) {
-                    // We previously reached the last clip.  Nothing
-                    // else to read.
-                    _queue_event(bd, (BD_EVENT){BD_EVENT_END_OF_TITLE, 0});
-                    return 0;
-                }
                 if (clip_pkt >= st->clip->end_pkt) {
 
                     // split read()'s at clip boundary
