@@ -168,6 +168,12 @@ typedef struct bd_title_info {
     BLURAY_TITLE_CHAPTER *chapters;
 } BLURAY_TITLE_INFO;
 
+typedef struct bd_sound_effect {
+    uint8_t         num_channels; /* 1 - mono, 2 - stereo */
+    uint32_t        num_frames;
+    const int16_t  *samples;      /* 48000 Hz, 16 bit LPCM. interleaved if stereo */
+} BLURAY_SOUND_EFFECT;
+
 /**
  *  Get library version
  *
@@ -220,7 +226,7 @@ void bd_free_title_info(BLURAY_TITLE_INFO *title_info);
 /**
  *  Initializes libbluray objects
  *
- * @param device_path   path to mounted Blu-ray disc
+ * @param device_path   path to mounted Blu-ray disc or device
  * @param keyfile_path  path to KEYDB.cfg (may be NULL)
  * @return allocated BLURAY object, NULL if error
  */
@@ -401,6 +407,14 @@ uint64_t bd_tell_time(BLURAY *bd);
  * Disc info
  */
 
+/* AACS error codes */
+#define BD_AACS_CORRUPTED_DISC  -1
+#define BD_AACS_NO_CONFIG       -2
+#define BD_AACS_NO_PK           -3
+#define BD_AACS_NO_CERT         -4
+#define BD_AACS_CERT_REVOKED    -5
+#define BD_AACS_MMC_FAILED      -6
+
 typedef struct {
     uint8_t  bluray_detected;
 
@@ -419,6 +433,14 @@ typedef struct {
     uint8_t  libbdplus_detected;
     uint8_t  bdplus_handled;
 
+    /* aacs error code */
+    int      aacs_error_code;
+    /* aacs MKB version */
+    int      aacs_mkbv;
+
+    /* Disc ID */
+    uint8_t  disc_id[20];
+
 } BLURAY_DISC_INFO;
 
 /**
@@ -480,7 +502,7 @@ typedef enum {
     BD_EVENT_ANGLE,     /* current angle, 1...N */
     BD_EVENT_TITLE,     /* current title, 1...N (0 = top menu) */
     BD_EVENT_PLAYLIST,  /* current playlist (xxxxx.mpls) */
-    BD_EVENT_PLAYITEM,  /* current play item */
+    BD_EVENT_PLAYITEM,  /* current play item, 0...N-1  */
     BD_EVENT_CHAPTER,   /* current chapter, 1...N */
     BD_EVENT_END_OF_TITLE,
 
@@ -507,6 +529,14 @@ typedef enum {
     /* Still playback for n seconds (reached end of still mode play item) */
     BD_EVENT_STILL_TIME,             /* 0 = infinite ; 1...300 = seconds */
 
+    /* Play sound effect */
+    BD_EVENT_SOUND_EFFECT,           /* effect ID */
+
+    /* Pop-Up menu available */
+    BD_EVENT_POPUP,                  /* 0 - no, 1 - yes */
+
+    /* Interactive menu visible */
+    BD_EVENT_MENU,                   /* 0 - no, 1 - yes */
 } bd_event_e;
 
 typedef struct {
@@ -624,6 +654,17 @@ int bd_user_input(BLURAY *bd, int64_t pts, uint32_t key);
  */
 int bd_mouse_select(BLURAY *bd, int64_t pts, uint16_t x, uint16_t y);
 
+/**
+ *
+ *  Get sound effect
+ *
+ * @param bd  BLURAY object
+ * @param effect_id  sound effect id (0...N)
+ * @param effect     sound effect data
+ * @return <0 when no effects, 0 when id out of range, 1 on success
+ */
+int bd_get_sound_effect(BLURAY *bd, unsigned sound_id, struct bd_sound_effect *effect);
+
 /*
  *
  */
@@ -634,9 +675,9 @@ struct meta_dl;
  *  Get meta information about the bluray disc.
  *
  * @param bd  BLURAY object
- * @return allocated META_DL (disclib) object, NULL on error
+ * @return META_DL (disclib) object, NULL on error
  */
-struct meta_dl *bd_get_meta(BLURAY *bd);
+const struct meta_dl *bd_get_meta(BLURAY *bd);
 
 
 struct clpi_cl;
